class Pettanr.Views.Sheet.PlayModule
class Pettanr.Views.Sheet.PlayModule.Header extends Pettanr.Views.Show.Header
  
  initialize: (options) ->
    super(options)
  
class Pettanr.Views.Sheet.PlayModule.Owner extends Pettanr.Views.Show.Owner
  
  initialize: (options) ->
    super(options)
  
class Pettanr.Views.Sheet.PlayModule.Footer extends Backbone.View
  tagName: 'table'
  className: 'no-border'
  
  initialize: (options) ->
    @panel = options.panel
    @sheet_panel = options.sheet_panel
  
  render: () ->
    this.$el.html('')
    this.$el.append(
      (new Pettanr.Views.Panel.Icon({item: @panel, half: true})).render().el
    )
    this.$el.append(
      (new Pettanr.Views.Author.Icon({item: @panel.author(), half: true})).render().el
    )
    this.$el.append(Pettanr.AppHelper.distance_of_time_in_words_to_now(@panel.get('updated_at')))
    #
    this.$el.append(
      (new Pettanr.Views.Panel.Icon({item: @sheet_panel, half: true})).render().el
    )
    this.$el.append(
      (new Pettanr.Views.Author.Icon({item: @panel.author(), half: true})).render().el
    )
    this.$el.append(Pettanr.AppHelper.distance_of_time_in_words_to_now(@sheet_panel.get('updated_at')))
    #
    if @panel.is_own()
      this.$el.append('')
    this
  
class Pettanr.Views.Sheet.PlayModule.Credits extends Backbone.View
  tagName: 'div'
  className: 'credits'
  
  initialize: (options) ->
    @parent = options.parent
    @licensed_pictures = {}
  
  render: () ->
    this.$el.html('')
    this
  
  push: (append_pictures) ->
    _.each append_pictures, (retriever, pid) =>
      return if @licensed_pictures[pid]
      @listenTo(retriever, 'retrieve', @retrieve_picture)
      retriever.retrieve()
  
  retrieve_picture: (picture) ->
    @licensed_pictures[pid] = picture
    credit = picture.credit_view()
    this.$el.append(credit.render().el)
  
class Pettanr.Views.Sheet.PlayModule.Panel extends Backbone.View
  tagName: 'div'
  
  initialize: (options) ->
    @sheet_panel = options.sheet_panel
  
  clear: () ->
    this.$el.html('')
    retriever = @sheet_panel.panel()
    @listenTo(retriever, 'retrieve', @retrieve_panel)
    retriever.retrieve()
    this
  
  retrieve_panel: (@panel) ->
    @render()
  
  render: () ->
    this.$el.html('')
    body = new Pettanr.Views.Panel.Body({
      panel: @panel,
      spot: null
    })
    this.$el.append(body.render().el)
    footer = new Pettanr.Views.Panel.Footer({
      sheet_panel: sheet_panel,
      panel: @panel,
    })
    this.$el.append(footer.render().el)
    this.credits.push(@panel.licensed_pictures())
    this
  
class Pettanr.Views.Sheet.PlayModule.Panels extends Backbone.View
  tagName: 'div'
  
  initialize: (options) ->
    @parent = options.parent
    @items = options.items
    @credits = options.credits
  
  render: () ->
    this.$el.html('')
    _.each @items, (sheet_panel) =>
      return if not sheet_panel.has_panel()
      panel_view = new Pettanr.Views.Sheet.PlayModule.Panel({
        sheet_panel: sheet_panel,
      })
      this.$el.append(panel_view.render().el)
    this
  
class Pettanr.Views.Sheet.PlayModule.Body extends Backbone.View
  tagName: 'div'
  
  initialize: (options) ->
    @parent = options.parent
    @list = options.list
    @items = @list.items()
    if @items.length < 1
      @render_empty()
    else
      @render()
  
  render_empty: () ->
    this.$el.html('')
    this.$el.append((new Tag.H2({content: I18n.t('sheets.play.empty')})).render().el)
    this
  
  render: () ->
    this.$el.html('')
    # paginate(@pager)
    credits = new Pettanr.Views.Sheet.PlayModule.Credits({parent: this})
    panels = new Pettanr.Views.Sheet.PlayModule.Panels({
      parent: this,
      items: @items,
      credits: credits,
    })
    this.$el.append(panels.render().el)
    this.$el.append(credits.render().el)
    rb = new Tag.RowBreak()
    this.$el.append(rb.render().el)
    # paginate(@pager)
    this
  
class Pettanr.Views.Sheet.Play extends Backbone.View
  tagName: 'div'
  className: 'show'
  
  initialize: (options) ->
    @item = options.item
    @list = options.list
    icon_url = Pettanr.url(@item.table_name(), 'show', {id: @item.get('id')})
    caption_url = Pettanr.url(@item.table_name(), 'play', {id: @item.get('id')})
    prof_url = Pettanr.url(@item.table_name(), 'show', {id: @item.get('id'), format: 'prof'})
    @header = new Pettanr.Views.Sheet.PlayModule.Header({
      item: @item, 
      caption: @item.get('caption'), 
      icon_url: icon_url,
      caption_url: caption_url,
      prof_url: prof_url
    })
    @author = new Pettanr.Views.Show.HeaderAuthor({item: @item})
    @body = new Pettanr.Views.Sheet.PlayModule.Body({
      parent: this,
      list: @list, 
    })
    @owner = new Pettanr.Views.Sheet.PlayModule.Owner({item: @item})
  
  render: () ->
    this.$el.html('')
    this.$el.append(@header.render().el)
    this.$el.append(@author.render().el)
    this.$el.append(@body.render().el)
    this.$el.append(@owner.render().el) if @item.is_own()
    this
  
