class Pettanr.Views.SpeechBalloon.NewElementLauncher extends Editor.EditorModule.DockModule.NewElementLauncher
  
  initialize: (options) ->
    super(options)
    @links_conf = {}
    _.extend(@links_conf, {
      template: {
        caption: 'Template',
        params: {
          controller: @model.pick_model().path_name(), action: 'index',
          page_size: 6, format: 'html'
        }
      }
    })
    _.extend(@links_conf, {
      inspire_recent: {
        caption: "Inspire recent item",
        params: {
          controller: @model.path_name(), action: 'index',
          page_size: 6, format: 'html'
        }
      }
    })
    if Pettanr.cache.operators.author
      _.extend(@links_conf, {
        inspire_my_recent: {
          caption: 'Inspire my recent item',
          params: {
            controller: @model.path_name(), action: 'by_author', id: Pettanr.cache.operators.author.get('id'), 
            page_size: 6, format: 'html'
          }
        }
      })
  
  render: () ->
    super()
    @add_links()
    this
  
  click_template: () ->
    @click_default(@links_conf.template.params)
  
  click_inspire_recent: () ->
    @click_default(@links_conf.inspire_recent.params)
  
  click_inspire_my_recent: () ->
    @click_default(@links_conf.inspire_my_recent.params)
  
class Pettanr.Views.SpeechBalloon.NewElement extends Backbone.View
  
  initialize: (options) ->
    @parent = options.parent
    @target_model = options.target_model
    @pick_model = @target_model.pick_model()
  
  start: (@dialog, params) ->
    @dialog.start(@target_model, params, null)
    @listenTo(@dialog, 'pick', @pick)
    @trigger('dialog:open', @dialog)
  
  pick: (item) ->
    if item.item_name() == @target_model.item_name()
      # Inspire
      @listenTo(this, 'ready', @ready)
      balloon_list = item.has_one('balloon')
      speech_list = item.has_one('speech')
      balloon_list.open(() =>
        balloon = balloon_list.items()[0]
        speech_list.open(() =>
          speech = speech_list.items()[0]
          new_item = new @target_model(item.attributes)
          new_item.unset('id', {silent: true})
          new_item.set({panel_id: @editor().item.get('id')}, {silent: true})
          balloon.unset('id', {silent: true})
          balloon.unset('speech_balloon_id', {silent: true})
          speech.unset('id', {silent: true})
          speech.unset('speech_balloon_id', {silent: true})
          new_item.boosts 'post'
          balloon.boosts 'post'
          speech.boosts 'post'
          new_item.set({balloon: balloon, speech: speech}, {silent: true})
          @trigger('ready', new_item)
        )
      )
    else if @target_model.is_traceable(item.item_name())
      # Trace
      tracer_class = @target_model['TraceFrom' + item.singular()]
      tracer = new tracer_class()
      @listenTo(tracer, 'trace', @trace)
      tracer.trace(item)
    else if item.item_name() == @target_model.pick_item_name()
      # Pick
      @listenTo(this, 'ready', @ready)
      speech_balloon_template = item
      writing_format_list = Locmare.ListGroup.list(
        'writing_formats', 'index', {}
      )
      writing_format_list.open(() =>
        writing_format = writing_format_list.items()[0]
        templates = {
          speech_balloon_template: speech_balloon_template,
          writing_format: writing_format
        }
        new_item = new @target_model()
        new_item.templates = templates
        new_item.set(new_item.pick(), {silent: true})
        new_item.boosts 'post'
        new_item.supply_default()
        # prepare balloon
        balloon = new Pettanr.Balloon()
        balloon.templates = templates
        balloon.set(balloon.pick(), {silent: true})
        balloon.boosts 'post'
        balloon.supply_default()
        # prepare speech
        speech = new Pettanr.Speech()
        speech.templates = templates
        speech.set(speech.pick(), {silent: true})
        speech.boosts 'post'
        speech.supply_default()
        # join elements
        new_item.set({balloon: balloon, speech: speech}, {silent: true})
        @trigger('ready', new_item)
      )
  
  ready: (new_item) ->
    @dialog.stop()
    @trigger('pick', new_item)
  
  trace: (speech_balloon) ->
    @dialog.http_get(speech_balloon.show_url())
  
  dock: () ->
    @parent.dock()
  
  editor: () ->
    @dock().editor()
  
