#サポートツール
require 'rubygems'
require 'active_support/all'
require 'rest_client'
require 'yaml'
require 'json'
require 'base64'

if ARGV.size < 1
  puts "uploader.rb src_host src_auth dest_host dest_auth"
  puts "upload on child dirs"
  exit
end

src_host = ARGV[0]
src_auth = ARGV[1]
dest_host = ARGV[2]
dest_auth = ARGV[3]

class Importer
  def initialize src_host, src_auth, dest_host, dest_auth
    @src_host = src_host
    @src_auth = src_auth
    @dest_host = dest_host
    @dest_auth = dest_auth
    @src_licenses = {}
    self.licenses(@src_host).each do |l|
      @src_licenses[l['id'].to_i] = l
    end
    @dest_licenses = {}
    self.licenses(@dest_host).each do |l|
      @dest_licenses[l['name']] = l
    end
    @src_speech_balloon_templates = {}
    self.speech_balloon_templates(@src_host, @src_auth).each do |sb|
      @src_speech_balloon_templates[sb['id'].to_i] = sb
    end
    @dest_speech_balloon_templates = {}
    self.speech_balloon_templates(@dest_host, @dest_auth).each do |sb|
      @dest_speech_balloon_templates[sb['name']] = sb
    end
    @src_writing_formats = {}
    self.writing_formats(          @dest_host, @dest_auth).each do |wf|
  #  self.writing_formats(@src_host, @src_auth).each do |wf|
      @src_writing_formats[wf['id'].to_i] = wf
    end
    @dest_writing_formats = {}
    self.writing_formats(@dest_host, @dest_auth).each do |wf|
      @dest_writing_formats[wf['name']] = wf
    end
    @src_system_pictures = {}
    self.system_pictures(@src_host, @src_auth).each do |sp|
      @src_system_pictures[sp['id'].to_i] = sp
    end
    @dest_system_pictures = {}
    self.system_pictures(@dest_host, @dest_auth).each do |sp|
      @dest_system_pictures[sp['md5']] = sp
    end
  end
  
  def licenses host
    res = {}
    u = host + 'licenses.json'
    r = RestClient.get(u, :params => {:page_size  => 100})
    JSON.parse r
  end
  
  def license_groups host
    res = {}
    u = host + 'license_groups.json'
    r = RestClient.get(u, :params => {:page_size  => 100})
    ary = JSON.parse r
    ary.each do |l|
      res[l['id'].to_i] = l
    end
    res
  end
  
  def speech_balloon_templates host, auth
    res = {}
    u = host + 'speech_balloon_templates.json'
    r = RestClient.get(u, :params => {:page_size  => 100, :auth_token => auth})
    JSON.parse r
  end
  
  def writing_formats host, auth
    res = {}
    u = host + 'writing_formats.json'
    r = RestClient.get(u, :params => {:page_size  => 100, :auth_token => auth})
    JSON.parse r
  end
  
  def system_pictures host, auth
    res = {}
    u = host + 'system_pictures.json'
    r = RestClient.get(u, :params => {:page_size  => 100, :auth_token => auth})
    JSON.parse r
  end
  
  def panels host, auth
    res = {}
    u = host + 'home/panels.json'
    r = RestClient.get(u, :params => {:page_size  => 100, :auth_token => auth})
    ary = JSON.parse r
    ary.each do |panel|
      res[panel['id'].to_i] = panel
    end
    res
  end
  
  def resource_picture md5, host, auth
    res = {}
    u = host + 'pictures/search.json'
    r = RestClient.get(u, :params => {:md5 => md5, :auth_token => auth})
    JSON.parse r
  end
  
  def each_panel
    self.panels(@src_host, @src_auth).each do |key, panel_json|
      puts 'source panel:' + key.to_s
      # 今はAPIの仕様が変わっているので、そのうち書き直す
      elements = panel_json['elements']
      panel_json.delete 'elements'
      c = true
      elements.each do |e|
        if e['link']
          element_name = 'panel_pictures'
          puts 'pp:' + e['id'].to_s
          rps = self.resource_picture e['picture']['md5'], @dest_host, @dest_auth
          if rps.any?
            rp = rps.first
            puts 'picture ok(' + rp['id'].to_s + ')'
          else
            puts 'no picture'
            c = false
            break
          end
          src_ls_name = @src_licenses[e['picture']['license_id']]['name'].gsub /License\:/, 'Licenses:'
          puts 'license:' + e['picture']['license_id'].to_s + '(' + src_ls_name + ')'
          if dl = @dest_licenses[src_ls_name]
            puts 'license ok(' + dl['id'].to_s + ')'
          else
            puts 'no license'
            c = false
            break
          end
          e['picture_id'] = rp['id']
          e.delete 'picture'
          e.delete 'license'
        elsif e['classname']
          element_name = 'speech_balloons'
          puts 'sb:' + e['id'].to_s
          src_sb_name = @src_speech_balloon_templates[e['speech_balloon_template_id']]['name']
          puts 'speech_balloon_template:' + e['speech_balloon_template_id'].to_s + '(' + src_sb_name + ')'
          if dsb = @dest_speech_balloon_templates[src_sb_name]
            puts 'speech_balloon_template ok(' + dsb['id'].to_s + ')'
          else
            puts 'no speech_balloon_template'
            c = false
            break
          end
          src_sp_md5 = @src_system_pictures[e['balloon']['system_picture_id']]['md5']
          puts 'balloon picture:' + e['balloon']['system_picture_id'].to_s + '(' + src_sp_md5 + ')'
          if dsp = @dest_system_pictures[src_sp_md5]
            puts 'balloon picture ok(' + dsp['id'].to_s + ')'
          else
            puts 'no balloon picture'
            c = false
            break
          end
          src_wf_name = @src_writing_formats[e['speech']['writing_format_id']]['name']
          puts 'speech writing_format:' + e['speech']['writing_format_id'].to_s + '(' + src_wf_name + ')'
          if dwf = @dest_writing_formats[src_wf_name]
            puts 'speech writing_format ok(' + dwf['id'].to_s + ')'
          else
            puts 'no speech writing_format'
            c = false
            break
          end
          e['speech_balloon_template_id'] = dsb['id']
          e["speech_balloon_template_settings"] = e["settings"]
          e.delete 'settings'
          e["speech_balloon_template_classname"] = dsb["classname"]
          e.delete 'classname'
          #e.delete 'speech_balloon_template'
          balloon = e['balloon']
          e.delete 'balloon'
          balloon.delete 'id'
          balloon.delete 'speech_balloon_id'
          balloon["speech_balloon_template_settings"] = e["settings"]
          balloon.delete 'settings'
          balloon['system_picture_id'] = dsp['id']
          balloon['speech_balloon_template_id'] = dsb['id']
          balloon["speech_balloon_template_classname"] = dsb["classname"]
          
          speech = e['speech']
          e.delete 'speech'
          speech.delete 'id'
          speech.delete 'speech_balloon_id'
          speech["speech_balloon_template_settings"] = e["settings"]
          speech.delete 'settings'
          speech['speech_balloon_template_id'] = dsb['id']
          speech["speech_balloon_template_classname"] = dsb["classname"]
          speech['writing_format_id'] = dwf['id']
    #      speech["writing_format_classname"] = 'Pettanr' + dwf["classname"]
          speech["writing_format_classname"] = dwf["classname"]
          
          e['balloon_attributes'] = balloon
          e['speech_attributes'] = speech
        elsif e['code']
          element_name = 'ground_colors'
          puts 'gc:' + e['id'].to_s
        elsif e['repeat']
          element_name = 'ground_pictures'
          puts 'gp:' + e['id'].to_s
          rps = self.resource_picture e['picture']['md5'], @dest_host, @dest_auth
          if rps.any?
            rp = rps.first
            puts 'picture ok(' + rp['id'].to_s + ')'
          else
            puts 'no picture'
            c = false
            break
          end
          #e['license_id'] = dl['id']
          e['picture_id'] = rp['id']
          #e.delete 'license'
        else
          c = false
          break
        end
        eid = e['id']
        e.delete 'id'
        e.delete 'panel_id'
        panel_json[element_name + '_attributes'] ||= {}
        panel_json[element_name + '_attributes']['new' + eid.to_s] = e
      end
      next unless c
      puts '============ importable'
      panel_json.delete 'id'
      panel_json.delete 'author'
      panel_json.delete 'author_id'
      panel_json.delete "created_at"
      panel_json.delete 'updated_at'
      yield panel_json
    end
  end
  
  def each_opset 
    self.each_subdir do |filename|
      imgfile = nil
      lsname = nil
      formname = nil
      attr  = nil
      Dir.glob(filename + '/*') do |fn|
        ext = File.extname(fn).downcase
        case ext
        when '.json'
          json = JSON.parse(File.open(fn).read)
          lsname = json["license_name"]
          formname = json["formname"]
          attr = json["attributes"]
        when '.png', '.gif', '.jpeg'
          imgfile = fn
        end
      end
      if imgfile and lsname and formname and attr
        yield imgfile, lsname, formname, attr
      end
    end
  end
  
  def upload fn
    oid = nil
    b = Base64.encode64(File.open(fn, 'rb').read)
    u = @host + 'original_pictures'
    begin
      r = RestClient.post(u, 
        {:original_picture => {:file  => b}, :auth_token => @auth}.to_json, 
        :content_type => :json, :accept => :json
      )
      o = JSON.parse r
      oid = o['id']
    rescue
      puts ' failed: ' + r.to_s
    end
    oid
  end
  
  def publish oid, lgid, rp
    res = nil
    u = @host + 'resource_pictures'
    attr = {
      :original_picture_id => oid, 
      :original_picture_license_group => {
        :original_picture_id => oid, :license_group_id => lgid
      }, 
      :resource_picture => {
        :original_picture_id => oid, 
        :license_id => rp['license_id'], 
        :artist_name => rp['artist_name'], 
        :system_picture_id => rp['system_picture_id'], 
        :license_group_classname => rp['license_group_classname'], 
        :license_group_settings => rp['license_group_settings'], 
        :credit_picture_settings => rp['credit_picture_settings'], 
        :license_settings => rp['license_settings']
      }, 
      :auth_token => @auth 
    }
    begin
      r = RestClient.post(u, 
        attr.to_json,
        :content_type => :json, :accept => :json
      )
      res = JSON.parse r
    rescue
      puts ' failed: ' + r.to_s
    end
    res
  end
  
  def publish_all
    each_opset do |imgfile, lsname, formname, attr|
      puts 'uploading: ' + imgfile
      oid = self.upload imgfile
      next unless oid
      puts 'success: ' + oid.to_s
      ls = @licenses[lsname]
      unless ls
        puts 'not detect licenses: ' + lsname.to_s
        next
      end
      lsid = ls['id']
      lgid = ls['license_group_id'].to_i
      lg = @license_groups[lgid]
      en = lg['classname']
      puts 'building: ' + formname
      rp = self.resource_picture oid, lgid, lsid, en, formname, attr
      next unless rp
      puts 'publishing: ' + lsname
      r = self.publish oid, lgid, rp
      puts 'success: ' + r['id'].to_s
    end
  end
  
end

u = Importer.new src_host, src_auth, dest_host, dest_auth

u.each_panel do |panel_json|
    u = dest_host + 'panels'
 File.open('/Sites/panel_create.json', 'w') do |f|
   f.write JSON.pretty_generate(panel_json)
 end
    begin
      r = RestClient.post(u, 
        {:json => panel_json, :auth_token => dest_auth}.to_json, 
        :content_type => :json, :accept => :json
      )
      res = JSON.parse r
    rescue
      puts ' failed: ' + r.to_s
    end
   # break
end


