# -*- encoding: utf-8 -*-
require 'spec_helper'
#フキダシ枠
describe Balloon do
  include ManifestsMacros
  before do
    @model = Balloon
  end
  
  describe 'ペタナイズに於いて' do
    it 'フキダシの要素として利用されるエレメントである' do
      expect(@model.ancestors.include?(Peta::Element)).to eq true
    end
    it '親モデルはフキダシである' do
      expect(@model.my_peta.parent_model_name).to eq 'speech_balloon'
    end
    describe 'ブーストに於いて' do
      before do
        @boosts = @model.my_peta.boost
        @boosts_names = ['speech_balloon_template']
      end
      it 'これらのブーストが定義されている' do
        set_boosts? @boosts, @boosts_names
      end
      describe 'フキダシテンプレートに於いて' do
        before do
          @mani = @boosts['speech_balloon_template']
        end
        it 'システム起動前に初期化可能なサポート機能が動作している' do
          expect(@mani.level).to eq 'post'
          expect(@mani.template_name).to eq 'speech_balloon_template'
          expect(@mani.column_name_for_template_module).to eq 'speech_balloon_template_module_name'
          expect(@mani.settings_column_name).to eq 'speech_balloon_template_settings'
          expect(@mani.foreign_key).to eq 'speech_balloon_template_id'
        end
        it 'システム起動後にしか初期化できないサポート機能が動作している' do
          expect(@mani.item_name_for_extend_model).to eq 'balloon'
          expect(@mani.setter_method_name).to eq 'balloon_extend='
          expect(@mani.getter_method_name).to eq 'balloon_extend'
          expect(@mani.extend_model_class_name).to eq 'Balloon'
          expect(@mani.extend_module_name).to eq 'BalloonModule'
        end
      end
    end
    
  end
  
  describe 'コントローラに於いて' do
    before do
      @controller = @model.my_controller
    end
    it 'フキダシ枠のためのコントローラである' do
      expect(@model.item_name).to eq 'balloon'
    end
    describe 'actionに於いて' do
      before do
        @actions = @controller.actions
        @actions_names = ['index', 'by_author', 'show', 
          'count', 'count_by_author', 
          'new', 'edit']
      end
      it 'これらのアクションが定義されている' do
        set_actions? @actions, @actions_names
      end
      describe 'baseに於いて' do
      end
      describe 'indexに於いて' do
        before do
          @action = @actions['index']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # フキダシ枠の公開リストを返すように定義している
          set_list_action? @action, 'balloon', 'public'
        end
      end
      describe 'by_authorに於いて' do
        before do
          @action = @actions['by_author']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # フキダシ枠の作家フィルタリストを返すように定義している
          set_list_action? @action, 'balloon', 'by_author'
        end
      end
      describe 'countに於いて' do
        before do
          @action = @actions['count']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          #フキダシ枠の公開リストのカウントを返すように定義している
          set_list_action? @action, 'balloon', 'public'
        end
      end
      describe 'count_by_authorに於いて' do
        before do
          @action = @actions['count_by_author']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # フキダシ枠の作家フィルタリストのカウントを返すように定義している
          set_list_action? @action, 'balloon', 'by_author'
        end
      end
      describe 'newに於いて' do
        before do
          @action = @actions['new']
        end
        it 'newタイプのアクションである' do
          expect(@action.type).to eq 'new'
        end
        it 'type' do
          expect(@action.item_name).to eq 'balloon'
        end
      end
      describe 'editに於いて' do
        before do
          @action = @actions['edit']
        end
        it 'editタイプのアクションである' do
          expect(@action.type).to eq 'edit'
        end
        it 'type' do
          expect(@action.item_name).to eq 'balloon'
        end
      end
    end
    
  end
  
  describe 'モデルに於いて' do
    before do
      @my_manifest = @model.my_manifest
    end
    it 'サポート機能が動作している' do
      expect(@my_manifest.model_name).to eq 'balloon'
      expect(@my_manifest.classify).to eq @model
      expect(@my_manifest.table_name).to eq 'balloons'
    end
    
    describe 'associationsに於いて' do
      describe 'belongs_toに於いて' do
        before do
          @belongs_to = @my_manifest.associations.belongs_to
          @names = ['speech_balloon', 'system_picture', 'speech_balloon_template']
        end
        it 'これらのモデルに所属している' do
          set_associations? @belongs_to, @names
        end
        describe 'speech_balloonに於いて' do
          before do
            @mani = @belongs_to['speech_balloon']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'speech_balloon'
            expect(@mani.id_column).to eq 'speech_balloon_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq SpeechBalloon
          end
        end
        describe 'system_pictureに於いて' do
          before do
            @mani = @belongs_to['system_picture']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'system_picture'
            expect(@mani.id_column).to eq 'system_picture_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq SystemPicture
          end
        end
        describe 'speech_balloon_templateに於いて' do
          before do
            @mani = @belongs_to['speech_balloon_template']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'speech_balloon_template'
            expect(@mani.id_column).to eq 'speech_balloon_template_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq SpeechBalloonTemplate
          end
        end
      end
      describe 'has_manyに於いて' do
        before do
          @has_many = @model.my_manifest.associations.has_many
          @names = []
        end
        it 'これらのモデルを所持している' do
          set_associations? @has_many, @names
        end
      end
    end
    
    describe 'attributesに於いて' do
      before do
        @attributes = @model.my_manifest.attributes
      end
      it 'これらのカラムを定義している' do
        set_attributes? @attributes, 
          ['id', 'speech_balloon_id', 'speech_balloon_template_id', 'speech_balloon_template_module_name', 
          'system_picture_id', 'x', 'y', 'width', 'height', 'r', 
          'speech_balloon_template_settings', 
          'created_at', 'updated_at']
      end
      describe 'speech_balloon_idに於いて' do
        before do
          @attribute = @attributes['speech_balloon_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'speech_balloon_template_idに於いて' do
        before do
          @attribute = @attributes['speech_balloon_template_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'speech_balloon_template_module_nameに於いて' do
        before do
          @attribute = @attributes['speech_balloon_template_module_name']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'system_picture_idに於いて' do
        before do
          @attribute = @attributes['system_picture_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'xに於いて' do
        before do
          @attribute = @attributes['x']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'yに於いて' do
        before do
          @attribute = @attributes['y']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'widthに於いて' do
        before do
          @attribute = @attributes['width']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'heightに於いて' do
        before do
          @attribute = @attributes['height']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'rに於いて' do
        before do
          @attribute = @attributes['r']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'speech_balloon_template_settingsに於いて' do
        before do
          @attribute = @attributes['speech_balloon_template_settings']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
    end
    
  end
  
end
