# -*- encoding: utf-8 -*-
require 'spec_helper'
#スクコマ
describe ScrollPanel do
  before do
    @admin = FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    SpeechBalloonTemplate.delete_all
    @speech_balloon_template = FactoryGirl.create :speech_balloon_template, "name" => "circle@pettan.com", "classname" => "CircleSpeechBalloon", "caption" => "cc",  "system_picture_id" => @sp.id, "settings" => '{}'
    @writing_format = FactoryGirl.create :writing_format
    @user = FactoryGirl.create( :user_yas)
    @author = FactoryGirl.create :author, :user_id => @user.id
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = FactoryGirl.create :author, :user_id => @other_user.id
    @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
    @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
    @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
  end
  
  describe '検証に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.build :scroll_panel, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @scroll_panel.t = 0
        @scroll_panel.should be_valid
      end
      it '上限データが通る' do
        @scroll_panel.t = 99999
        @scroll_panel.should be_valid
      end
    end
    
    context 'scroll_idを検証するとき' do
      it 'nullなら失敗する' do
        @scroll_panel.scroll_id = nil
        @scroll_panel.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @scroll_panel.scroll_id = 'a'
        @scroll_panel.should_not be_valid
      end
      it '存在するスクロールでなければ失敗する' do
        @scroll_panel.scroll_id = 0
        @scroll_panel.should_not be_valid
      end
    end
    
    context 'panel_idを検証するとき' do
      it 'nullなら失敗する' do
        @scroll_panel.panel_id = nil
        @scroll_panel.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @scroll_panel.panel_id = 'a'
        @scroll_panel.should_not be_valid
      end
      it '存在するコマでなければ失敗する' do
        @scroll_panel.panel_id = 0
        @scroll_panel.should_not be_valid
      end
    end
    
    context 'tを検証するとき' do
      it 'nullなら失敗する' do
        @scroll_panel.t = nil
        @scroll_panel.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @scroll_panel.t = 'a'
        @scroll_panel.should_not be_valid
      end
      it '負なら失敗する' do
        @scroll_panel.t = -1
        @scroll_panel.should_not be_valid
      end
    end
    
    context 'author_idを検証するとき' do
      it 'nullなら失敗する' do
        @scroll_panel.author_id = nil
        @scroll_panel.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @scroll_panel.author_id = 'a'
        @scroll_panel.should_not be_valid
      end
      it '存在する作家でなければ失敗する' do
        @scroll_panel.author_id = 0
        @scroll_panel.should_not be_valid
      end
    end
    context '全体を検証するとき' do
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
    end
    
    #dbのデフォルト値が0だから明示的にnilにしないと追加ができない
    it 'tをnilにする' do
      @scroll_panel = FactoryGirl.build :scroll_panel, :scroll_id => @scroll.id, :panel_id => @panel.id
      @scroll_panel.supply_default
      @scroll_panel.t.should be_nil
    end
    
  end
  
  describe '上書き補充に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
    end
    
    context 'author_idを補充' do
      it '問答無用でauthor_idを補充する' do
        @scroll_panel = FactoryGirl.build :scroll_panel, :scroll_id => @scroll.id, :panel_id => @panel.id
        @scroll_panel.author_id = nil
        @scroll_panel.overwrite @author
        @scroll_panel.author_id.should eq @author.id
      end
    end
    
  end
  
  describe '所持判定に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
      @scrollo = FactoryGirl.create :scroll, :author_id => @other_author.id
      @panelo = FactoryGirl.create :panel, :author_id => @other_author.id
      @scroll_panelo = FactoryGirl.create :scroll_panel, :author_id => @other_author.id, :scroll_id => @scrollo.id, :panel_id => @panelo.id
    end
    context '事前チェックする' do
      it '自身にロールリストからの作家取得を依頼している' do
        ScrollPanel.should_receive(:get_author_from_roles).with(any_args).exactly(1)
        r = @scroll_panel.own?([@author])
      end
    end
    context 'ロール内作家が取得できるとき' do
      before do
      end
      it 'ロール内作家のidが自身の作家idと一致するなら許可する' do
        ScrollPanel.stub(:get_author_from_roles).with(any_args).and_return(@author)
        r = @scroll_panel.own?([@author])
        r.should be_true
      end
      it 'ロール内作家のidが自身の作家idと一致しないならno' do
        ScrollPanel.stub(:get_author_from_roles).with(any_args).and_return(@other_author)
        @scroll_panel.own?(@other_author).should be_false
      end
    end
    context 'ロール内作家が取得できないとき' do
      before do
        ScrollPanel.stub(:get_author_from_roles).with(any_args).and_return(nil)
      end
      it 'Falseを返す' do
        r = @scroll_panel.own?([@author])
        r.should be_false
      end
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
    end
    context 'オープンモードのとき' do
      before do
        MagicNumber['run_mode'] = 0
      end
      it '自身にゲスト用ロールチェックを問い合わせしている' do
        ScrollPanel.any_instance.stub(:guest_role_check).and_return(true)
        ScrollPanel.any_instance.should_receive(:guest_role_check).with(any_args).exactly(1)
        r = @scroll_panel.visible?([@author])
      end
      it 'ゲスト用ロールチェックが失敗したとき、falseを返す' do
        ScrollPanel.any_instance.stub(:guest_role_check).and_return(false)
        r = @scroll_panel.visible?([@author])
        r.should be_false
      end
    end
    context 'クローズドモードのとき' do
      before do
        MagicNumber['run_mode'] = 1
      end
      it '自身に読者用ロールチェックを問い合わせしている' do
        ScrollPanel.any_instance.stub(:reader_role_check).and_return(true)
        ScrollPanel.any_instance.should_receive(:reader_role_check).with(any_args).exactly(1)
        r = @scroll_panel.visible?([@author])
      end
      it '読者用ロールチェックが失敗したとき、falseを返す' do
        ScrollPanel.any_instance.stub(:reader_role_check).and_return(false)
        r = @scroll_panel.visible?([@author])
        r.should be_false
      end
    end
    context '事前チェックする' do
      before do
        MagicNumber['run_mode'] = 1
        ScrollPanel.any_instance.stub(:reader_role_check).and_return(true)
      end
      it '自身のスクロールに所持判定を問い合わせしている' do
        Scroll.any_instance.stub(:own?).and_return(true)
        Scroll.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = @scroll_panel.visible?([@author])
      end
      it '自身のスクロールに閲覧許可を問い合わせしている' do
        Scroll.any_instance.stub(:own?).and_return(false)
        Scroll.any_instance.stub(:visible?).and_return(true)
        Scroll.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = @scroll_panel.visible?([@author])
      end
    end
    context 'つつがなく終わるとき' do
      before do
        MagicNumber['run_mode'] = 1
        ScrollPanel.any_instance.stub(:reader_role_check).and_return(true)
      end
      it '自分のスクロールのスクコマなら許可する' do
        Scroll.any_instance.stub(:own?).and_return(true)
        Scroll.any_instance.stub(:visible).and_return(0)
        r = @scroll_panel.visible?([@author])
        r.should be_true
      end
      it '他人の非公開スクロールのスクコマなら許可しない' do
        Scroll.any_instance.stub(:own?).and_return(false)
        Scroll.any_instance.stub(:visible).and_return(0)
        r = @scroll_panel.visible?([@author])
        r.should be_false
      end
      it '他人のスクロールのスクコマでも公開なら許可する' do
        Scroll.any_instance.stub(:own?).and_return(false)
        Scroll.any_instance.stub(:visible).and_return(1)
        r = @scroll_panel.visible?([@author])
        r.should be_true
      end
    end
  end
  
  describe 'プレイリスト取得に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
      @panel2 = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      @other_scroll = FactoryGirl.create :scroll, :author_id => @other_author.id, :visible => 1
      @other_panel = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
    end
    context 'offset補正について' do
      it '文字列から数値に変換される' do
        ScrollPanel.offset(100, '8').should eq 8
      end
      it 'nilの場合は0になる' do
        ScrollPanel.offset(100).should eq 0
      end
      #投稿されたコマ数以上の値が指定されたときは、最後のコマだけになる
      #最後のコマとは、コマ数‐1．
      it '1件のときオフセット1なら0になる' do
        ScrollPanel.offset(1, '1').should eq 0
      end
      it '5件のときオフセット5なら4になる' do
        ScrollPanel.offset(5, '5').should eq 4
      end
      # 負の値が指定されたときは、最後のコマから数えてコマを飛ばして表示する。
      #-4のときは、最後から4つのコマを表示する。 
      it '2件のときオフセット-1なら1になる' do
        ScrollPanel.offset(2, '-1').should eq 1
      end
      it '5件のときオフセット-2なら3になる' do
        ScrollPanel.offset(5, '-2').should eq 3
      end
      # 最終的なが負になるなど、不正な値が入ったときは0となる。 
      it '2件のときオフセット-5なら0になる' do
        ScrollPanel.offset(2, '-5').should eq 0
      end
    end
    context 'panel_count補正について' do
      it '文字列から数値に変換される' do
        ScrollPanel.panel_count(100, '7').should eq 7
      end
      it 'nilの場合はScrollPanel.default_panel_sizeになる' do
        ScrollPanel.panel_count(100).should eq ScrollPanel.default_panel_size
      end
      it '0以下の場合はScrollPanel.default_panel_sizeになる' do
        ScrollPanel.panel_count(100, '0').should eq ScrollPanel.default_panel_size
      end
      it 'ScrollPanel.max_panel_sizeを超えた場合はScrollPanel.max_panel_sizeになる' do
        ScrollPanel.panel_count(100, '1000').should eq ScrollPanel.max_panel_size
      end
    end
    it 'リストを返す' do
      c = ScrollPanel.play_list @scroll, @author
      c.should eq [@scroll_panel]
    end
    it 't順で並んでいる' do
      #公開スクロールの公開コマは(他人のスクロールであっても)含んでいる
      v = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 1
      c = ScrollPanel.play_list @scroll, @author
      c.should eq [ @scroll_panel, v]
    end
    it '非公開のコマは含んでいる' do
      h = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel2.id, :t => 1
      c = ScrollPanel.play_list @scroll, @author
      c.should eq [ @scroll_panel, h]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 1
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 2
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 3
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 4
      end
      it 'offset=0なら末尾2件を返す' do
        #時系列で並んでいる
        c = ScrollPanel.play_list( @scroll, @author, 0, 2)
        c.should eq [@scroll_panel, @scroll_panel2]
      end
      it 'offset=2なら中間2件を返す' do
        c = ScrollPanel.play_list(@scroll, @author, 2, 2)
        c.should eq [@scroll_panel3, @scroll_panel4]
      end
      it 'offset=4なら先頭1件を返す' do
        c = ScrollPanel.play_list(@scroll, @author, 4, 2)
        c.should eq [@scroll_panel5]
      end
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id, :visible => 1
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
      @other_scroll = FactoryGirl.create :scroll, :author_id => @other_author.id, :visible => 1
      @other_panel = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      @hidden_scroll = FactoryGirl.create :scroll, :author_id => @author.id, :visible => 0
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        ScrollPanel.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        ScrollPanel.page().should eq 1
      end
      it '0以下の場合は1になる' do
        ScrollPanel.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        ScrollPanel.page_size('7').should eq 7
      end
      it 'nilの場合はScrollPanel.default_page_sizeになる' do
        ScrollPanel.page_size().should eq ScrollPanel.default_page_size
      end
      it '0以下の場合はScrollPanel.default_page_sizeになる' do
        ScrollPanel.page_size('0').should eq ScrollPanel.default_page_size
      end
      it 'ScrollPanel.max_page_sizeを超えた場合はScrollPanel.max_page_sizeになる' do
        ScrollPanel.page_size('1000').should eq ScrollPanel.max_page_size
      end
    end
    it 'リストを返す' do
      c = ScrollPanel.list
      c.should eq [@scroll_panel]
    end
    it '時系列で並んでいる' do
      #公開スクロールのScrollPanelは(他人のScrollPanelであっても)含んでいる
      v = FactoryGirl.create :scroll_panel, :author_id => @other_author.id, :scroll_id => @other_scroll.id, :panel_id => @other_panel.id, :t => 0, :updated_at => Time.now + 100
      c = ScrollPanel.list 
      c.should eq [ v, @scroll_panel]
    end
    it '非公開のスクコマは(自分のスクコマであっても)含まない' do
      h = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @hidden_scroll.id, :panel_id => @panel.id, :t => 0
      c = ScrollPanel.list 
      c.should eq [ @scroll_panel]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 1, :updated_at => Time.now + 100
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 2, :updated_at => Time.now + 200
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 3, :updated_at => Time.now + 300
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 4, :updated_at => Time.now + 400
      end
      it '通常は2件を返す' do
        l = ScrollPanel.list 1, 2
        l.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        l = ScrollPanel.list 1, 2
        l.should eq [@scroll_panel5, @scroll_panel4]
      end
      it 'page=2なら中間2件を返す' do
        l = ScrollPanel.list 2, 2
        l.should eq [@scroll_panel3, @scroll_panel2]
      end
      it 'page=3なら先頭1件を返す' do
        l = ScrollPanel.list 3, 2
        l.should eq [@scroll_panel]
      end
    end
  end
  
  describe '自分のスクコマ一覧取得に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id, :visible => 1
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
      @other_scroll = FactoryGirl.create :scroll, :author_id => @other_author.id, :visible => 1
      @other_panel = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      @hscroll = FactoryGirl.create :scroll, :author_id => @author.id, :visible => 0
      @hpanel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        ScrollPanel.stub(:list_opt).with(any_args).and_return({})
        ScrollPanel.should_receive(:list_opt).with(any_args).exactly(1)
        r = ScrollPanel.mylist @author
      end
    end
    it 'リストを返す' do
      s = ScrollPanel.mylist @author
      s.should eq [@scroll_panel]
    end
    it '時系列で並んでいる' do
      ns = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 1, :updated_at => Time.now + 100
      sl = ScrollPanel.mylist @author
      sl.should eq [ns, @scroll_panel]
    end
    it '他人のスクコマはコマスクロールともに公開でも含まない' do
      so = FactoryGirl.create :scroll_panel, :author_id => @other_author.id, :scroll_id => @other_scroll.id, :panel_id => @other_panel.id
      sl = ScrollPanel.mylist @author
      sl.should eq [@scroll_panel]
    end
    it '自分のスクコマはコマスクロールともに非公開でも含んでいる' do
      hs = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @hscroll.id, :panel_id => @hpanel.id, :updated_at => Time.now + 100
      sl = ScrollPanel.mylist @author
      sl.should eq [hs, @scroll_panel]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 1, :updated_at => Time.now + 100
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 2, :updated_at => Time.now + 200
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 3, :updated_at => Time.now + 300
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 4, :updated_at => Time.now + 400
        ScrollPanel.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        l = ScrollPanel.mylist @author, 1, 2
        l.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        l = ScrollPanel.mylist @author, 1, 2
        l.should eq [@scroll_panel5, @scroll_panel4]
      end
      it 'page=2なら中間2件を返す' do
        l = ScrollPanel.mylist @author, 2, 2
        l.should eq [@scroll_panel3, @scroll_panel2]
      end
      it 'page=3なら先頭1件を返す' do
        l = ScrollPanel.mylist @author, 3, 2
        l.should eq [@scroll_panel]
      end
    end
  end
  
  describe '他作家のスクコマ一覧取得に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id, :visible => 1
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
      @other_scroll = FactoryGirl.create :scroll, :author_id => @other_author.id, :visible => 1
      @other_panel = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      @other_scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @other_author.id, :scroll_id => @other_scroll.id, :panel_id => @other_panel.id
    end
    it 'リストを返す' do
      r = ScrollPanel.himlist @other_author
      r.should eq [@other_scroll_panel]
    end
    it '時系列で並んでいる' do
      ns = FactoryGirl.create :scroll_panel, :author_id => @other_author.id, :scroll_id => @other_scroll.id, :panel_id => @other_panel.id, :t => 1, :updated_at => Time.now + 100
      r = ScrollPanel.himlist @other_author
      r.should eq [ns, @other_scroll_panel]
    end
    it '公開スクロールのスクコマ' do
      @hscroll = FactoryGirl.create :scroll, :author_id => @other_author.id, :visible => 0
      ns = FactoryGirl.create :scroll_panel, :author_id => @other_author.id, :scroll_id => @hscroll.id, :panel_id => @other_panel.id, :t => 1, :updated_at => Time.now + 100
      r = ScrollPanel.himlist @other_author
      r.should eq [@other_scroll_panel]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @other_scroll_panel2 = FactoryGirl.create :scroll_panel, :author_id => @other_author.id, :scroll_id => @other_scroll.id, :panel_id => @other_panel.id, :t => 1, :updated_at => Time.now + 100
        @other_scroll_panel3 = FactoryGirl.create :scroll_panel, :author_id => @other_author.id, :scroll_id => @other_scroll.id, :panel_id => @other_panel.id, :t => 2, :updated_at => Time.now + 200
        @other_scroll_panel4 = FactoryGirl.create :scroll_panel, :author_id => @other_author.id, :scroll_id => @other_scroll.id, :panel_id => @other_panel.id, :t => 3, :updated_at => Time.now + 300
        @other_scroll_panel5 = FactoryGirl.create :scroll_panel, :author_id => @other_author.id, :scroll_id => @other_scroll.id, :panel_id => @other_panel.id, :t => 4, :updated_at => Time.now + 400
        ScrollPanel.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        l = ScrollPanel.himlist @other_author, 1, 2
        l.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        l = ScrollPanel.himlist @other_author, 1, 2
        l.should eq [@other_scroll_panel5, @other_scroll_panel4]
      end
      it 'page=2なら中間2件を返す' do
        l = ScrollPanel.himlist @other_author, 2, 2
        l.should eq [@other_scroll_panel3, @other_scroll_panel2]
      end
      it 'page=3なら先頭1件を返す' do
        l = ScrollPanel.himlist @other_author, 3, 2
        l.should eq [@other_scroll_panel]
      end
    end
  end
  
  describe 'スクコマ一覧ページ制御に於いて' do
    before do
      ScrollPanel.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = ScrollPanel.list_paginate 
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it 'スクコマ一覧の取得条件を利用している' do
      ScrollPanel.stub(:list_where).with(any_args).and_return('')
      ScrollPanel.should_receive(:list_where).with(any_args).exactly(1)
      r = ScrollPanel.list_paginate 
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = ScrollPanel.list_paginate 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe '自分のスクコマ一覧ページ制御に於いて' do
    before do
      ScrollPanel.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = ScrollPanel.mylist_paginate @author
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it '自分のスクコマ一覧の取得条件を利用している' do
      ScrollPanel.stub(:mylist_where).with(any_args).and_return('')
      ScrollPanel.should_receive(:mylist_where).with(any_args).exactly(1)
      r = ScrollPanel.mylist_paginate @author
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = ScrollPanel.mylist_paginate @author, 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe '他作家のスクコマ一覧ページ制御に於いて' do
    before do
      ScrollPanel.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = ScrollPanel.himlist_paginate @other_author
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it '他作家のスクコマ一覧の取得条件を利用している' do
      ScrollPanel.stub(:himlist_where).with(any_args).and_return('')
      ScrollPanel.should_receive(:himlist_where).with(any_args).exactly(1)
      r = ScrollPanel.himlist_paginate @other_author
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = ScrollPanel.himlist_paginate @other_author, 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe 'list関連テーブルプションに於いて' do
    it '3つの項目を含んでいる' do
      r = ScrollPanel.list_opt
      r.should have(3).items
    end
    it 'スクロールを含んでいる' do
      r = ScrollPanel.list_opt
      r.has_key?(:scroll).should be_true
    end
      it 'スクロールは作家を含んでいる' do
        r = ScrollPanel.list_opt
        r[:scroll].has_key?(:author).should be_true
      end
    it '作家を含んでいる' do
      r = ScrollPanel.list_opt
      r.has_key?(:author).should be_true
    end
    it 'コマを含んでいる' do
      r = ScrollPanel.list_opt
      r.has_key?(:panel).should be_true
    end
      it 'コマは作家を含んでいる' do
        r = ScrollPanel.list_opt
        r[:panel].has_key?(:author).should be_true
      end
      it 'コマはコマ絵を含んでいる' do
        r = ScrollPanel.list_opt
        r[:panel].has_key?(:panel_pictures).should be_true
      end
        it 'コマ絵は実素材を含んでいる' do
          r = ScrollPanel.list_opt
          r[:panel][:panel_pictures].has_key?(:picture).should be_true
        end
          it '実素材は絵師を含んでいる' do
            r = ScrollPanel.list_opt
            r[:panel][:panel_pictures][:picture].has_key?(:artist).should be_true
          end
          it '実素材はライセンスを含んでいる' do
            r = ScrollPanel.list_opt
            r[:panel][:panel_pictures][:picture].has_key?(:license).should be_true
          end
      it 'コマはフキダシを含んでいる' do
        r = ScrollPanel.list_opt
        r[:panel].has_key?(:speech_balloons).should be_true
      end
        it 'フキダシはフキダシ枠を含んでいる' do
          r = ScrollPanel.list_opt
          r[:panel][:speech_balloons].has_key?(:balloon).should be_true
        end
        it 'フキダシはセリフを含んでいる' do
          r = ScrollPanel.list_opt
          r[:panel][:speech_balloons].has_key?(:speech).should be_true
        end
  end
  describe 'json一覧出力オプションに於いて' do
  end
  
  describe '単体取得に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        ScrollPanel.stub(:show_opt).with(any_args).and_return({})
        ScrollPanel.should_receive(:show_opt).with(any_args).exactly(1)
        r = ScrollPanel.show @scroll_panel.id, @author
      end
      it '閲覧許可を問い合わせている' do
        ScrollPanel.any_instance.stub(:visible?).with(@author).and_return(true)
        ScrollPanel.any_instance.should_receive(:visible?).with(@author).exactly(1)
        r = ScrollPanel.show @scroll_panel.id, @author
      end
    end
    it '指定のスクコマを返す' do
      l = ScrollPanel.show @scroll_panel.id, @author
      l.should eq @scroll_panel
    end
    context '他人のスクコマを開こうとしたとき' do
      it '403Forbidden例外を返す' do
        ScrollPanel.any_instance.stub(:visible?).with(@other_author).and_return(false)
        lambda{
          ScrollPanel.show @scroll_panel.id, @other_author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないスクコマを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          ScrollPanel.show 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  
  describe '編集取得に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        ScrollPanel.stub(:show_opt).with(any_args).and_return({})
        ScrollPanel.should_receive(:show_opt).with(any_args).exactly(1)
        r = ScrollPanel.edit @scroll_panel.id, @author
      end
      it '所持判定を問い合わせている' do
        ScrollPanel.any_instance.stub(:own?).with(any_args).and_return(true)
        ScrollPanel.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = ScrollPanel.edit @scroll_panel.id, @author
      end
    end
    it '指定のスクコマを返す' do
      l = ScrollPanel.edit @scroll_panel.id, @author
      l.should eq @scroll_panel
    end
    context '他人のスクコマを開こうとしたとき' do
      it '403Forbidden例外を返す' do
        ScrollPanel.any_instance.stub(:own?).and_return(false)
        lambda{
          ScrollPanel.edit @scroll_panel.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないスクコマを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          ScrollPanel.edit 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  
  describe '単体取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = ScrollPanel.show_opt
      r.has_key?(:include).should be_true
    end
    it '3つの項目を含んでいる' do
      r = ScrollPanel.show_opt[:include]
      r.should have(3).items
    end
    it 'スクロールを含んでいる' do
      r = ScrollPanel.show_opt[:include]
      r.has_key?(:scroll).should be_true
    end
      it 'スクロールは作家を含んでいる' do
        r = ScrollPanel.show_opt[:include]
        r[:scroll].has_key?(:author).should be_true
      end
    it '作家を含んでいる' do
      r = ScrollPanel.show_opt[:include]
      r.has_key?(:author).should be_true
    end
    it 'コマを含んでいる' do
      r = ScrollPanel.show_opt[:include]
      r.has_key?(:panel).should be_true
    end
      it 'コマは作家を含んでいる' do
        r = ScrollPanel.show_opt[:include]
        r[:panel].has_key?(:author).should be_true
      end
      it 'コマはコマ絵を含んでいる' do
        r = ScrollPanel.show_opt[:include]
        r[:panel].has_key?(:panel_pictures).should be_true
      end
        it 'コマ絵は実素材を含んでいる' do
          r = ScrollPanel.show_opt[:include]
          r[:panel][:panel_pictures].has_key?(:picture).should be_true
        end
          it '実素材は絵師を含んでいる' do
            r = ScrollPanel.show_opt[:include]
            r[:panel][:panel_pictures][:picture].has_key?(:artist).should be_true
          end
          it '実素材はライセンスを含んでいる' do
            r = ScrollPanel.show_opt[:include]
            r[:panel][:panel_pictures][:picture].has_key?(:license).should be_true
          end
      it 'コマはフキダシを含んでいる' do
        r = ScrollPanel.show_opt[:include]
        r[:panel].has_key?(:speech_balloons).should be_true
      end
        it 'フキダシはフキダシ枠を含んでいる' do
          r = ScrollPanel.show_opt[:include]
          r[:panel][:speech_balloons].has_key?(:balloon).should be_true
        end
        it 'フキダシはセリフを含んでいる' do
          r = ScrollPanel.show_opt[:include]
          r[:panel][:speech_balloons].has_key?(:speech).should be_true
        end
  end
  describe 'json単体取得オプションに於いて' do
  end
  
  describe 'スクコマのjson出力に於いて' do
    before do
      #コマを作成しておく。
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @pp = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 1, :width => @p.width, :height => @p.height
      @sb = FactoryGirl.build :speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @speech_balloon_template.id
      @speech = @sb.build_speech(
        FactoryGirl.attributes_for(:speech, :writing_format_id => @writing_format.id)
      )
      @balloon = @sb.build_balloon(
        FactoryGirl.attributes_for(:balloon, :system_picture_id => @sp.id)
      )
      @sb.boost
      @sb.save!
      @gc = @panel.ground_colors.create(
        FactoryGirl.attributes_for(:ground_color, :panel_id => @panel.id)
      )
      @gp = @panel.ground_pictures.create(
        FactoryGirl.attributes_for(:ground_picture, :panel_id => @panel.id, :picture_id => @p.id)
      )
      @panel.reload
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
    end
    context '事前チェックする' do
      before do
        Panel.any_instance.stub(:elements).and_return('{}')
      end
      it 'コマ要素のjson出力を依頼している' do
        Panel.any_instance.stub(:visible?).with(any_args).and_return(true)
        Panel.any_instance.should_receive(:elements).with(any_args).exactly(1)
        r = @scroll_panel.scroll_panel_as_json @author
      end
    end
    it 'json textを返している' do
      r = JSON.parse @scroll_panel.scroll_panel_as_json(@author)
      r.is_a?(Hash).should be_true
    end
    it 'scroll,author,panel,コマ要素を含んでいる' do
      r = JSON.parse @scroll_panel.scroll_panel_as_json(@author)
      r.has_key?('scroll').should be_true
      r['scroll'].has_key?('author').should be_true
      r.has_key?('author').should be_true
      r.has_key?('panel').should be_true
      r['panel'].has_key?('author').should be_true
    end
    context 'コマ閲覧許可のとき' do
      before do
        Panel.any_instance.stub(:visible?).with(any_args).and_return(true)
      end
      it 'コマ要素にコマ要素を追加している' do
        r = JSON.parse @scroll_panel.scroll_panel_as_json(@author)
        r['panel'].has_key?('elements').should be_true
        r['panel']['elements'].should_not be_empty
      end
    end
    context 'コマ閲覧不許可のとき' do
      before do
        Panel.any_instance.stub(:visible?).with(any_args).and_return(false)
      end
      it 'コマ要素にデータを含ませない' do
        r = JSON.parse @scroll_panel.scroll_panel_as_json(@author)
        r['panel'].has_key?('elements').should be_false
      end
    end
  end
  
  describe 'スクコマリストのjson出力に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
      ScrollPanel.any_instance.stub(:scroll_panel_as_json).with(@author).and_return('{"s": 5}')
    end
    context 'つつがなく終わるとき' do
      it 'スクコマのjson出力を依頼している' do
        ScrollPanel.any_instance.should_receive(:scroll_panel_as_json).with(@author).exactly(1)
        r = ScrollPanel.list_as_json_text [@scroll_panel], @author
      end
    end
    it 'json textを返している' do
      r = ScrollPanel.list_as_json_text [@scroll_panel], @author
      j = JSON.parse r
      j.is_a?(Array).should be_true
    end
    it 'スクコマを含んでいる' do
      r = ScrollPanel.list_as_json_text [@scroll_panel], @author
      j = JSON.parse r
      j.first.has_key?('s').should be_true
    end
  end
  
  describe 'ライセンス素材に於いて' do
    before do
      #コマを作成しておく。
      @op2 = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p2 = FactoryGirl.create :picture, :original_picture_id => @op2.id, :license_id => @license.id, :artist_id => @artist.id
      @rp2 = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op2.id, :picture_id => @p2.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @pp = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 0, :width => @p.width, :height => @p.height, :picture_id => @p.id
      @panel.reload
      @panel2 = FactoryGirl.create :panel, :author_id => @author.id
      @pp2= FactoryGirl.create :panel_picture, :panel_id => @panel2.id, :t => 0, :width => @p2.width, :height => @p2.height, :picture_id => @p2.id
      @panel2.reload
      
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
      @scroll_panel2 = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel2.id
    end
    context '事前チェック' do
    end
    context 'つつがなく終わるとき' do
    end
    it '連想配列(値は実素材、キーは実素材id)を返している' do
      r = ScrollPanel.licensed_pictures [@scroll_panel, @scroll_panel2]
      r.is_a?(Hash).should be_true
      r.should have(2).items
      r[@pp.picture_id].should eq @p
      r[@pp2.picture_id].should eq @p2
    end
    context 'コマが削除されているときき' do
      before do
        @panel2.destroy
      end
      it '削除されているコマは無視する' do
        r = ScrollPanel.licensed_pictures [@scroll_panel, @scroll_panel2]
        r.is_a?(Hash).should be_true
        r.should have(1).items
        r[@pp.picture_id].should eq @p
        r[@pp2.picture_id].should be_nil
      end
    end
  end
  
  describe 't補充値に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
    end
    
    context 'スクロール初のコマなら' do
      it '0を補充値とする' do
        @scroll_panel = FactoryGirl.build :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
        @scroll_panel.t = nil
        r = ScrollPanel.new_t @scroll_panel.scroll_id
        r.should eq 0
      end
    end
    context 'スクロールに一個コマがあるとき' do
      it '1を補充値とする' do
        FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 0
        @scroll_panel = FactoryGirl.build :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
        @scroll_panel.t = nil
        r = ScrollPanel.new_t @scroll_panel.scroll_id
        r.should eq 1
      end
    end
    context 'スクロールに2個コマがあるとき' do
      it '2を補充値とする' do
        FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 0
        FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id, :t => 1
        @scroll_panel = FactoryGirl.build :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
        @scroll_panel.t = nil
        r = ScrollPanel.new_t @scroll_panel.scroll_id
        r.should eq 2
      end
    end
  end
  describe 'シリアライズチェックに於いて' do
    context 'つつがなく終わるとき' do
      it '0からシリアライズされているならTrueを返す' do
        r = ScrollPanel.serial? [0, 1, 2]
        r.should be_true
      end
      it '見た目はシリアライズされてなくてもソート結果が無事ならtrueを返す' do
        r = ScrollPanel.serial? [0, 2, 1]
        r.should be_true
      end
      it '見た目はシリアライズされてなくてもソート結果が無事ならtrueを返す' do
        r = ScrollPanel.serial? [ 2, 1, 4, 3, 0]
        r.should be_true
      end
    end
    context '異常なとき' do
      it '0から始まらないならFalseを返す' do
        r = ScrollPanel.serial? [1, 2, 3]
        r.should be_false
      end
      it '連続していないならFalseを返す' do
        r = ScrollPanel.serial? [0, 1, 2, 4]
        r.should be_false
      end
      it '連続していないならFalseを返す' do
        r = ScrollPanel.serial? [0, 1, 2, 4, 5]
        r.should be_false
      end
    end
  end
  describe 't収集に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id
      @c2scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll2.id, :panel_id => @panel.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it 'スクコマから同一スクロールのtだけを収集している' do
        r = ScrollPanel.collect_t @scroll_panel
        r.should eq [0]
      end
    end
    context '複数コマのとき' do
      it 'スクコマから同一スクロールのtだけを収集している' do
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        r = ScrollPanel.collect_t @scroll_panel
        r.sort.should eq [0, 1]
      end
    end
    context '複数コマでヨソのスクロールも混じっているとき' do
      it 'スクコマから同一スクロールのtだけを収集している' do
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        r = ScrollPanel.collect_t @scroll_panel
        r.sort.should eq [0, 1]
      end
    end
  end
  describe 'tチェックに於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.build :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it 't収集を依頼している' do
        ScrollPanel.should_receive(:collect_t).with(any_args).exactly(1)
        ScrollPanel.stub(:collect_t).with(any_args).and_return([])
        ScrollPanel.stub(:serial?).with(any_args).and_return(true)
        r = ScrollPanel.validate_t @scroll_panel
      end
      it '収集したtをシリアライズチェック依頼している' do
        ScrollPanel.stub(:collect_t).with(any_args).and_return([])
        ScrollPanel.should_receive(:serial?).with(any_args).exactly(1)
        ScrollPanel.stub(:serial?).with(any_args).and_return(true)
        r = ScrollPanel.validate_t @scroll_panel
      end
    end
    #実データでチェック
    #依頼チェックだけでは不安なので最低限のチェックを
    context '新規のとき' do
      it '一件だけで正常通過している' do
        @scroll_panel = FactoryGirl.build :scroll_panel, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id, :t => 0
        r = ScrollPanel.validate_t @scroll_panel
        r.should be_true 
      end
    end
    context '既存のとき' do
      it '2件目を作っても正常通過している' do
        @scroll_panel = FactoryGirl.create :scroll_panel, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id, :t => 0
        @scroll_panel2 = FactoryGirl.build :scroll_panel, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id, :t => 1
        r = ScrollPanel.validate_t @scroll_panel2
        r.should be_true 
      end
    end
  end
  describe '挿入シフトに於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
    end
    context '依頼チェック' do
      #テーブルが空で0に挿入
      it 'Updateを依頼している' do
        ScrollPanel.stub(:update_all).with(any_args)
        ScrollPanel.should_receive(:update_all).with(any_args).exactly(1)
        @scroll_panel = FactoryGirl.build :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel.insert_shift
      end
    end
    context 'テーブルに1件(t:0)で0に挿入したとき' do
      before do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.build :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      end
      it '既存の行を1にシフトしている' do
        @scroll_panel2.insert_shift
        l = ScrollPanel.find :all
        l.first.t.should eq 1
      end
      it 'これから挿入するt(0)が欠番になっている' do
        @scroll_panel2.insert_shift
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.include?(0).should_not be_true
      end
    end
    context 'テーブルに2件(t:0,1)で1に挿入したとき' do
      before do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.build :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      end
      it '既存のt1を2にシフトしてこれから挿入するt(1)が欠番になっている' do
        @scroll_panel3.insert_shift
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 2]
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)で2に挿入したとき' do
      before do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel6 = FactoryGirl.build :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      end
      it '既存のt1を2にシフトしてこれから挿入するt(1)が欠番になっている' do
        @scroll_panel6.insert_shift
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 1, 3, 4, 5]
      end
    end
    context '先ほどのケース+他のスクロール1件で挿入したとき' do
      before do
        @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id
        @scroll_panelc2 = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll2.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel6 = FactoryGirl.build :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      end
      it '既存のt1を2にシフトしてこれから挿入するt(1)が欠番になっている' do
        @scroll_panel6.insert_shift
        l = ScrollPanel.find(:all, :conditions => ['scroll_id = ?', @scroll.id]).map {|s| s.t }
        l.sort.should eq [0, 1, 3, 4, 5]
      end
      it '他のスクロールに影響がない' do
        ot = @scroll_panelc2.t
        @scroll_panel6.insert_shift
        @scroll_panelc2.reload
        @scroll_panelc2.t.should eq ot
      end
    end
  end
  describe '少ない方に移動に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
    end
    context '依頼チェック' do
      it 'Updateを依頼している' do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        ScrollPanel.stub(:update_all).with(any_args)
        ScrollPanel.should_receive(:update_all).with(any_args).exactly(1)
        ot = @scroll_panel2.t
        @scroll_panel2.t = 0
        @scroll_panel2.lesser_shift ot
      end
    end
    context 'テーブルに2件(t:0,1)で1を0に移動したとき' do
      before do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel2.t
        @scroll_panel2.t = 0
      end
      it '既存のt0を1にシフトしてこれから挿入するt(0)が欠番になっている' do
        #移動させたい行はそのまま残る
        @scroll_panel2.lesser_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [1, 1]
      end
      it '既存のt0を1にシフトしている' do
        @scroll_panel2.lesser_shift @ot
        @scroll_panel.reload
        @scroll_panel.t.should eq 1
      end
    end
    context 'テーブルに3件(t:0,1,2)で2を1に移動したとき' do
      before do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel3.t
        @scroll_panel3.t = 1
      end
      it '既存のt1を2にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @scroll_panel3.lesser_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 2, 2]
      end
      it '既存のt1を2にシフトしている' do
        @scroll_panel3.lesser_shift @ot
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 2
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)で3を1に移動したとき' do
      before do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel4.t
        @scroll_panel4.t = 1
      end
      it 'これから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @scroll_panel4.lesser_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 2, 3, 3, 4]
      end
      it '既存のt0には変化がない' do
        @scroll_panel4.lesser_shift @ot
        @scroll_panel.reload
        @scroll_panel.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @scroll_panel4.lesser_shift @ot
        @scroll_panel5.reload
        @scroll_panel5.t.should eq 4
      end
      it '既存のt1を2にシフトしている' do
        @scroll_panel4.lesser_shift @ot
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 2
      end
      it '既存のt2を3にシフトしている' do
        @scroll_panel4.lesser_shift @ot
        @scroll_panel3.reload
        @scroll_panel3.t.should eq 3
      end
    end
    context '先ほどのケース+他のスクロール1件で挿入したとき' do
      before do
        @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id
        @scroll_panelc2 = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll2.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel4.t
        @scroll_panel4.t = 1
      end
      it 'これから挿入するt(1)が欠番になっている' do
        @scroll_panel4.lesser_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0, 2, 3, 3, 4]
      end
      it '既存のt0には変化がない' do
        @scroll_panel4.lesser_shift @ot
        @scroll_panel.reload
        @scroll_panel.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @scroll_panel4.lesser_shift @ot
        @scroll_panel5.reload
        @scroll_panel5.t.should eq 4
      end
      it '既存のt1を2にシフトしている' do
        @scroll_panel4.lesser_shift @ot
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 2
      end
      it '既存のt2を3にシフトしている' do
        @scroll_panel4.lesser_shift @ot
        @scroll_panel3.reload
        @scroll_panel3.t.should eq 3
      end
      it '他のスクロールに影響がない' do
        @scroll_panel4.lesser_shift @ot
        @scroll_panelc2.reload
        @scroll_panelc2.t.should eq 0
      end
    end
    #例外ケース。
    #負のときは0として正常扱い
    context 'テーブルに2件(t:0,1)で1を-1に移動したとき' do
      before do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel2.t
        @scroll_panel2.t = -1
      end
      it '既存のt0を1にシフトしてこれから挿入するt(0)が欠番になっている' do
        #移動させたい行はそのまま残る
        @scroll_panel2.lesser_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [1, 1]
      end
      it '既存のt0を1にシフトしている' do
        @scroll_panel2.lesser_shift @ot
        @scroll_panel.reload
        @scroll_panel.t.should eq 1
      end
      it '既存のt1は0に補正されている' do
        @scroll_panel2.lesser_shift @ot
        @scroll_panel2.t.should eq 0
      end
    end
  end
  describe '大きい方に移動に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
    end
    context '依頼チェック' do
      it 'Updateを依頼している' do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        ScrollPanel.stub(:update_all).with(any_args)
        ScrollPanel.should_receive(:update_all).with(any_args).exactly(1)
        ot = @scroll_panel.t
        @scroll_panel.t = 1
        @scroll_panel.higher_shift ot
      end
    end
    context 'テーブルに2件(t:0,1)で0を1に移動したとき' do
      before do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel.t
        @scroll_panel.t = 1
      end
      it '既存のt1を0にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @scroll_panel.higher_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0]
      end
      it '既存のt1を0にシフトしている' do
        @scroll_panel.higher_shift @ot
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 0
      end
    end
    context 'テーブルに3件(t:0,1,2)で0を1に移動したとき' do
      before do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel.t
        @scroll_panel.t = 1
      end
      it '既存のt1を0にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @scroll_panel.higher_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0, 2]
      end
      it '既存のt1を0にシフトしている' do
        @scroll_panel.higher_shift @ot
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 0
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)で1を3に移動したとき' do
      before do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel2.t
        @scroll_panel2.t = 3
      end
      it 'これから挿入するt(3)が欠番になっている' do
        #移動させたい行はそのまま残る
        @scroll_panel2.higher_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 1, 1, 2, 4]
      end
      it '既存のt0には変化がない' do
        @scroll_panel2.higher_shift @ot
        @scroll_panel.reload
        @scroll_panel.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @scroll_panel2.higher_shift @ot
        @scroll_panel5.reload
        @scroll_panel5.t.should eq 4
      end
      it '既存のt2を1にシフトしている' do
        @scroll_panel2.higher_shift @ot
        @scroll_panel3.reload
        @scroll_panel3.t.should eq 1
      end
      it '既存のt3を2にシフトしている' do
        @scroll_panel2.higher_shift @ot
        @scroll_panel4.reload
        @scroll_panel4.t.should eq 2
      end
    end
    context '先ほどのケース+他のスクロール1件で挿入したとき' do
      before do
        @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id
        @scroll_panelc2 = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll2.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel2.t
        @scroll_panel2.t = 3
      end
      it 'これから挿入するt(3)が欠番になっている' do
        #移動させたい行はそのまま残る
        @scroll_panel2.higher_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0, 1, 1, 2, 4]
      end
      it '既存のt0には変化がない' do
        @scroll_panel2.higher_shift @ot
        @scroll_panel.reload
        @scroll_panel.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @scroll_panel2.higher_shift @ot
        @scroll_panel5.reload
        @scroll_panel5.t.should eq 4
      end
      it '既存のt2を1にシフトしている' do
        @scroll_panel2.higher_shift @ot
        @scroll_panel3.reload
        @scroll_panel3.t.should eq 1
      end
      it '既存のt3を2にシフトしている' do
        @scroll_panel2.higher_shift @ot
        @scroll_panel4.reload
        @scroll_panel4.t.should eq 2
      end
      it '他のスクロールに影響がない' do
        @scroll_panel2.higher_shift @ot
        @scroll_panelc2.reload
        @scroll_panelc2.t.should eq 0
      end
    end
    #例外ケース。
    #max超えたときはmaxとして正常扱い
    context 'テーブルに2件(t:0,1)で0を2に移動したとき' do
      before do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel.t
        @scroll_panel.t = 2
      end
      it '既存のt1を0にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @scroll_panel.higher_shift @ot
        l = ScrollPanel.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0]
      end
      it '既存のt1を0にシフトしている' do
        @scroll_panel.higher_shift @ot
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 0
      end
      it '既存のt0は1に補正されている' do
        @scroll_panel.higher_shift @ot
        @scroll_panel.t.should eq 1
      end
    end
  end
  describe '入れ替えに於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
    end
    context '新tが旧tより小さいとき' do
      it '少ない方に移動を依頼している' do
        ScrollPanel.any_instance.stub(:lesser_shift).with(any_args)
        ScrollPanel.any_instance.should_receive(:lesser_shift).with(any_args).exactly(1)
        ot = @scroll_panel2.t
        @scroll_panel2.t = 0
        @scroll_panel2.update_shift ot
      end
    end
    context '新tが旧tより大きいとき' do
      it '大きい方に移動を依頼している' do
        ScrollPanel.any_instance.stub(:higher_shift).with(any_args)
        ScrollPanel.any_instance.should_receive(:higher_shift).with(any_args).exactly(1)
        ot = @scroll_panel.t
        @scroll_panel.t = 1
        @scroll_panel.update_shift ot
      end
    end
  end
  describe '順序入れ替えに於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
    end
    context 'オブジェクトが新規でtが空のとき' do
      it '末尾追加としてtを補充依頼している' do
        @scroll_panel = FactoryGirl.build :scroll_panel, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        ScrollPanel.stub(:new_t).with(any_args).and_return(0)
        ScrollPanel.should_receive(:new_t).with(any_args).exactly(1)
        @scroll_panel.t = nil
        r = @scroll_panel.rotate
      end
    end
    context 'オブジェクトが新規でtが設定されているとき' do
      it '挿入追加として挿入シフトを依頼している' do
        @scroll_panel = FactoryGirl.build :scroll_panel, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        ScrollPanel.any_instance.stub(:insert_shift).with(any_args)
        ScrollPanel.any_instance.should_receive(:insert_shift).with(any_args).exactly(1)
        @scroll_panel.t = 0
        r = @scroll_panel.rotate
      end
    end
    context 'オブジェクトが新規でなくtが設定されているとき' do
      it 'コマ移動として入れ替えを依頼している' do
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        ScrollPanel.any_instance.stub(:update_shift).with(any_args)
        ScrollPanel.any_instance.should_receive(:update_shift).with(1).exactly(1)
        @scroll_panel2.t = 0
        r = @scroll_panel.rotate 1
      end
    end
    context 'オブジェクトが新規でなくtが空のとき' do
      it '入れ替えもシフトもせず、tを空のままにしている' do
        #結果、tに欠番が生じてシリアライズチェックでひっかかる
      end
    end
  end
  describe '編集許可に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.build :scroll_panel, :t => nil, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it 'trueを返す' do
        r = @scroll_panel.allow?
        r.should be_true
      end
    end
    context 'スクロールで引っかかるとき' do
      it 'falseを返す' do
        Panel.any_instance.stub(:usable?).with(any_args).and_return(true)
        Scroll.any_instance.stub(:own?).with(any_args).and_return(false)
        r = @scroll_panel.allow?
        r.should be_false
      end
    end
    context 'コマで引っかかるとき' do
      it 'falseを返す' do
        Scroll.any_instance.stub(:own?).with(any_args).and_return(true)
        Panel.any_instance.stub(:usable?).with(any_args).and_return(false)
        r = @scroll_panel.allow?
        r.should be_false
      end
    end
    context 'スクロールまたはコマが指定されていなかったとき' do
      it 'nilを返す' do
        Scroll.any_instance.stub(:own?).with(any_args).and_return(true)
        @scroll_panel.panel_id = nil
        r = @scroll_panel.allow?
        r.should eq nil
      end
      it 'nilを返す' do
        Panel.any_instance.stub(:usable?).with(any_args).and_return(true)
        @scroll_panel.scroll_id = nil
        r = @scroll_panel.allow?
        r.should eq nil
      end
    end
  end
  describe '保存に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.build :scroll_panel, :t => nil, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '編集許可チェックを依頼している' do
        ScrollPanel.any_instance.stub(:allow?).with(any_args).and_return(true)
        ScrollPanel.any_instance.should_receive(:allow?).with(any_args).exactly(1)
        r = @scroll_panel.store
      end
      it '順序入れ替えを依頼している' do
        ScrollPanel.any_instance.stub(:rotate).with(any_args).and_return(0)
        ScrollPanel.any_instance.should_receive(:rotate).with(any_args).exactly(1)
        ScrollPanel.any_instance.stub(:save).with(any_args).and_return(true)
        ScrollPanel.stub(:validate_t).with(any_args).and_return(true)
        r = @scroll_panel.store 
      end
      it '保存を依頼している' do
        ScrollPanel.stub(:new_t).with(any_args).and_return(0)
        ScrollPanel.any_instance.stub(:save).with(any_args).and_return(true)
        ScrollPanel.any_instance.should_receive(:save).with(any_args).exactly(1)
        ScrollPanel.stub(:validate_t).with(any_args).and_return(true)
        r = @scroll_panel.store
      end
      it 'tのシリアライズチェックを依頼している' do
        ScrollPanel.stub(:new_t).with(any_args).and_return(0)
        ScrollPanel.any_instance.stub(:save).with(any_args).and_return(true)
        ScrollPanel.stub(:validate_t).with(any_args).and_return(true)
        ScrollPanel.should_receive(:validate_t).with(any_args).exactly(1)
        r = @scroll_panel.store
      end
    end
    #入れ替えテストと同じテストを実施。こちらはシフトだけでなく本尊も更新されている
    context 'テーブルに5件(t:0,1,2,3,4)+他のスクロール1件で2に挿入したとき' do
      before do
        @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id
        @scroll_panelc2 = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll2.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel6 = FactoryGirl.build :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      end
      it '既存のt0には変化がない' do
        @scroll_panel6.store
        @scroll_panel.reload
        @scroll_panel.t.should eq 0
      end
      it '既存のt1には変化がない' do
        @scroll_panel6.store
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 1
      end
      it '既存のt2を3にシフトしている' do
        @scroll_panel6.store
        @scroll_panel3.reload
        @scroll_panel3.t.should eq 3
      end
      it '既存のt3を4にシフトしている' do
        @scroll_panel6.store
        @scroll_panel4.reload
        @scroll_panel4.t.should eq 4
      end
      it '既存のt5を5にシフトしている' do
        @scroll_panel6.store
        @scroll_panel5.reload
        @scroll_panel5.t.should eq 5
      end
      it '新規のt2が作成されている' do
        @scroll_panel6.store
        @scroll_panel6.reload
        @scroll_panel6.t.should eq 2
      end
      it '他のスクロールに影響がない' do
        @ot = @scroll_panelc2.t
        @scroll_panel6.store
        @scroll_panelc2.reload
        @scroll_panelc2.t.should eq @ot
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)+他のスクロール1件で3を1に移動したとき' do
      before do
        @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id
        @scroll_panelc2 = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll2.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel4.t
        @scroll_panel4.t = 1
      end
      it '既存のt0には変化がない' do
        @scroll_panel4.store @ot
        @scroll_panel.reload
        @scroll_panel.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @scroll_panel4.store @ot
        @scroll_panel5.reload
        @scroll_panel5.t.should eq 4
      end
      it '既存のt1を2にシフトしている' do
        @scroll_panel4.store @ot
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 2
      end
      it '既存のt2を3にシフトしている' do
        @scroll_panel4.store @ot
        @scroll_panel3.reload
        @scroll_panel3.t.should eq 3
      end
      it '既存のt3を1にシフトしている' do
        @scroll_panel4.store @ot
        @scroll_panel4.reload
        @scroll_panel4.t.should eq 1
      end
      it '他のスクロールに影響がない' do
        @scroll_panel4.store @ot
        @scroll_panelc2.reload
        @scroll_panelc2.t.should eq 0
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)+他のスクロール1件で1を3に移動したとき' do
      before do
        @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id
        @scroll_panelc2 = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll2.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel2.t
        @scroll_panel2.t = 3
      end
      it '既存のt0には変化がない' do
        @scroll_panel2.store @ot
        @scroll_panel.reload
        @scroll_panel.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @scroll_panel2.store @ot
        @scroll_panel5.reload
        @scroll_panel5.t.should eq 4
      end
      it '既存のt1を3にシフトしている' do
        @scroll_panel2.store @ot
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 3
      end
      it '既存のt2を1にシフトしている' do
        @scroll_panel2.store @ot
        @scroll_panel3.reload
        @scroll_panel3.t.should eq 1
      end
      it '既存のt3を2にシフトしている' do
        @scroll_panel2.store @ot
        @scroll_panel4.reload
        @scroll_panel4.t.should eq 2
      end
      it '他のスクロールに影響がない' do
        @scroll_panel2.store @ot
        @scroll_panelc2.reload
        @scroll_panelc2.t.should eq 0
      end
    end
    #ロールバックテスト。入れ替えが直接DBをいじるので、すべてのケースで確実にロールバックを確認する
    context 'テーブルに5件(t:0,1,2,3,4)+他のスクロール1件で2に挿入したが保存に失敗したとき' do
      before do
        ScrollPanel.any_instance.stub(:save).with(any_args).and_return(false)
        @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id
        @scroll_panelc2 = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll2.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel6 = FactoryGirl.build :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      end
      it '既存のtに変化がない' do
        @scroll_panel6.store
        @scroll_panel.reload
        @scroll_panel.t.should eq 0
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 1
        @scroll_panel3.reload
        @scroll_panel3.t.should eq 2
        @scroll_panel4.reload
        @scroll_panel4.t.should eq 3
        @scroll_panel5.reload
        @scroll_panel5.t.should eq 4
        @scroll_panelc2.reload
        @scroll_panelc2.t.should eq 0
      end
      it 'falseを返す' do
        r = @scroll_panel6.store
        r.should be_false
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)+他のスクロール1件で3を1に移動したがシリアルチェックに失敗したとき' do
      before do
        ScrollPanel.stub(:validate_t).with(any_args).and_return(false)
        @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id
        @scroll_panelc2 = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll2.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @ot = @scroll_panel4.t
        @scroll_panel4.t = 1
      end
      it '既存のtに変化がない' do
        @scroll_panel4.store @ot
        @scroll_panel.reload
        @scroll_panel.t.should eq 0
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 1
        @scroll_panel3.reload
        @scroll_panel3.t.should eq 2
        @scroll_panel4.reload
        @scroll_panel4.t.should eq 3
        @scroll_panel5.reload
        @scroll_panel5.t.should eq 4
        @scroll_panelc2.reload
        @scroll_panelc2.t.should eq 0
      end
      it 'falseを返す' do
        r = @scroll_panel4.store @ot
        r.should be_false
      end
      it 'tにエラーメッセージが入っている' do
        @scroll_panel4.store @ot
        @scroll_panel4.errors[:t].should_not be_empty
        @scroll_panel4.valid?.should be_true
      end
    end
    context '編集不可だったとき' do
      before do
        @scroll_panel = FactoryGirl.build :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        ScrollPanel.any_instance.stub(:allow?).and_return(false)
      end
      it '403Forbidden例外を返す' do
        lambda{
          @scroll_panel.store
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
  end
  describe '切り詰め処理つき削除に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @scroll_panel = FactoryGirl.create :scroll_panel, :t => 0, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '削除される' do
        lambda{
          @scroll_panel.destroy_and_shorten
        }.should change(ScrollPanel, :count ).by(-1)
      end
      it 'Trueを返す' do
        r = @scroll_panel.destroy_and_shorten
        r.should be_true 
      end
    end
    context '削除に失敗したとき' do
      before do
        ScrollPanel.any_instance.stub(:destroy).and_return(false)
      end
      it 'ロールバックされる' do
        lambda{
          @scroll_panel.destroy_and_shorten
        }.should_not change(ScrollPanel, :count )
      end
      it 'Falseを返す' do
        r = @scroll_panel.destroy_and_shorten
        r.should be_false
      end
    end
    #連携テスト。切り詰めが直接DBをいじる
    context '2件で先頭を削除したとき' do
      before do
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      end
      it '行が削除される' do
        lambda{
          @scroll_panel.destroy_and_shorten
        }.should change(ScrollPanel, :count ).by(-1)
      end
      it '先頭は削除される' do
        @scroll_panel.destroy_and_shorten
        lambda{
          ScrollPanel.find @scroll_panel.id
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
      it '2件目は前に詰められる' do
        @scroll_panel.destroy_and_shorten
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 0
      end
    end
    context '3件で先頭を削除したとき' do
      before do
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      end
      it '行が削除される' do
        lambda{
          @scroll_panel.destroy_and_shorten
        }.should change(ScrollPanel, :count ).by(-1)
      end
      it '先頭は削除される' do
        @scroll_panel.destroy_and_shorten
        lambda{
          ScrollPanel.find @scroll_panel.id
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
      it '2件目は前に詰められる' do
        @scroll_panel.destroy_and_shorten
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 0
      end
      it '3件目は前に詰められる' do
        @scroll_panel.destroy_and_shorten
        @scroll_panel3.reload
        @scroll_panel3.t.should eq 1
      end
    end
    context '5件で3件目を削除したとき' do
      before do
        @scroll_panel2 = FactoryGirl.create :scroll_panel, :t => 1, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel3 = FactoryGirl.create :scroll_panel, :t => 2, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel4 = FactoryGirl.create :scroll_panel, :t => 3, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
        @scroll_panel5 = FactoryGirl.create :scroll_panel, :t => 4, :scroll_id => @scroll.id, :panel_id => @panel.id, :author_id => @author.id
      end
      it '行が削除される' do
        lambda{
          @scroll_panel3.destroy_and_shorten
        }.should change(ScrollPanel, :count ).by(-1)
      end
      it '1件目は変化がない' do
        @scroll_panel3.destroy_and_shorten
        @scroll_panel.reload
        @scroll_panel.t.should eq 0
      end
      it '2件目は変化がない' do
        @scroll_panel3.destroy_and_shorten
        @scroll_panel2.reload
        @scroll_panel2.t.should eq 1
      end
      it '3件目は削除される' do
        @scroll_panel3.destroy_and_shorten
        lambda{
          ScrollPanel.find @scroll_panel3.id
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
      it '4件目は前に詰められる' do
        @scroll_panel3.destroy_and_shorten
        @scroll_panel4.reload
        @scroll_panel4.t.should eq 2
      end
      it '5件目は前に詰められる' do
        @scroll_panel3.destroy_and_shorten
        @scroll_panel5.reload
        @scroll_panel5.t.should eq 3
      end
    end
    #ロールバックテスト。切り詰めが直接DBをいじるので、すべてのケースで確実にロールバックを確認する
  end
end
